# -*- coding: utf-8 -*-

from copy import deepcopy
from tools import trace, timer

from PyQt5.QtWidgets import (
    QMessageBox, QUndoCommand, QUndoStack,
)

from Model.Network.Node import Node
from Model.Network.Edge import Edge
from Model.Network.Graph import Graph

class SetCommand(QUndoCommand):
    def __init__(self, element, column, new_value):
        QUndoCommand.__init__(self)

        self._el = element
        self._column = column
        self._old = self._el[self._column]
        self._new = new_value

    def undo(self):
        self._el[self._column] = self._old

    def redo(self):
        self._el[self._column] = self._new

class EnableEdgeCommand(QUndoCommand):
    def __init__(self, edge, enable):
        QUndoCommand.__init__(self)

        self._edge = edge
        self._old = self._edge._enable
        self._enable = enable

    def undo(self):
        self._edge.enable(enable = self._old)

    def redo(self):
        self._edge.enable(enable = self._enable)

class ReverseEdgeCommand(QUndoCommand):
    def __init__(self, edge):
        QUndoCommand.__init__(self)

        self._edge = edge

    def undo(self):
        self._edge.reverse()

    def redo(self):
        self._edge.reverse()
